@@ -4,6 +4,9 @@
 # Univention AD Connector
 #  Basic class for the UCS connector part
 #
+# Like what you see? Join us!
+# https://www.univention.com/about-us/careers/vacancies/
+#
 # Copyright 2004-2022 Univention GmbH
 #
 # https://www.univention.de/
@@ -1358,9 +1361,18 @@ class ucs(object):
 							if attr not in object['changed_attributes']:
 								object['changed_attributes'].append(attr)
 					if not (set(object['changed_attributes']) - self.irrelevant_attributes):
-						ud.debug(ud.LDAP, ud.INFO, "sync_to_ucs: ignore %r" % (original_object['dn'],))
-						ud.debug(ud.LDAP, ud.ALL, "sync_to_ucs: changed_attributes=%s" % (object['changed_attributes'],))
-						return True
+						if property_type == "user" \
+						   and self.configRegistry.is_false('connector/ad/mapping/user/password/disabled', True) \
+						   and not self.configRegistry.is_true('connector/ad/mapping/user/password/kinit', False):
+							if object['attributes'].get('pwdLastSet', [b'1'])[0] == b'0':
+								ud.debug(ud.LDAP, ud.INFO, "sync_to_ucs: pwdLastSet is 0. Do not ignore %r" % (original_object['dn'],))
+							else:
+								ud.debug(ud.LDAP, ud.INFO, "sync_to_ucs: ignore %r" % (original_object['dn'],))
+								return True
+						else:
+							ud.debug(ud.LDAP, ud.INFO, "sync_to_ucs: ignore %r" % (original_object['dn'],))
+							ud.debug(ud.LDAP, ud.ALL, "sync_to_ucs: changed_attributes=%s" % (object['changed_attributes'],))
+							return True
 				else:
 					object['changed_attributes'] = list(original_attributes.keys())
 			ud.debug(ud.LDAP, ud.INFO, "The following attributes have been changed: %s" % object['changed_attributes'])
@@ -1369,13 +1381,14 @@ class ucs(object):
 
 			# Check if the object on UCS side should be synchronized
 			#  https://forge.univention.org/bugzilla/show_bug.cgi?id=37351
-			old_ucs_ldap_object = {}
-			old_ucs_ldap_object['dn'] = object.get('olddn', object['dn'])
-			old_ucs_ldap_object['attributes'] = self.get_ucs_ldap_object(old_ucs_ldap_object['dn'])
+			if property_type == "windowscomputer":
+				old_ucs_ldap_object = {}
+				old_ucs_ldap_object['dn'] = object.get('olddn', object['dn'])
+				old_ucs_ldap_object['attributes'] = self.get_ucs_ldap_object(old_ucs_ldap_object['dn'])
 
-			if old_ucs_ldap_object['attributes'] and self._ignore_object(property_type, old_ucs_ldap_object):
-				ud.debug(ud.LDAP, ud.PROCESS, 'The object %r will be ignored because a valid match filter for this object was not found.' % (old_ucs_ldap_object['dn'],))
-				return True
+				if old_ucs_ldap_object['attributes'] and self._ignore_object(property_type, old_ucs_ldap_object):
+					ud.debug(ud.LDAP, ud.PROCESS, 'The object %r will be ignored because _ignore_object matched the old object state in OpenLDAP.' % (old_ucs_ldap_object['dn'],))
+					return True
 
 			old_object = self.get_ucs_object(property_type, object.get('olddn', object['dn']))
 
